from django.contrib.auth.models import BaseUserManager


class PhoneOTPManager(BaseUserManager):
    """
    Manager implementation that uses:
        - phone
    """

    def create_user(self, phone, password, **extra_fields):
        """
        Creates usual user.
        """

        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_active", False)
        extra_fields.setdefault("is_superuser", False)

        return self._create(phone, password, **extra_fields)

    def create_superuser(self, phone, password, **extra_fields):
        """
        Creates super user.
        """
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_active", True)
        extra_fields.setdefault("is_superuser", True)

        return self._create(phone, password, **extra_fields)

    def _create(self, phone, password, **extra_fields):
        """
        Base method that implements user creation with email and password.
        """

        if not phone:
            raise ValueError("User should have phone number.")

        user = self.model(phone=phone, **extra_fields)

        user.set_password(password)
        user.save()

        return user



from swap_user.forms import BaseUserOptionalFieldsForm, BaseUserRequiredFieldsForm
from swap_user.to_phone.models import PhoneUser


class EditPhoneUserForm(BaseUserOptionalFieldsForm):
    """
    Form for - PhoneUser model.
    With optional `password_1` and `password_2` fields.

    Suitable for edit user pages.
    """

    class Meta(BaseUserOptionalFieldsForm.Meta):
        model = PhoneUser


class AddPhoneUserForm(BaseUserRequiredFieldsForm):
    """
    Form for - PhoneUser model.
    With required `password_1` and `password_2` fields.

    Suitable for user add pages.
    """

    class Meta(BaseUserRequiredFieldsForm.Meta):
        model = PhoneUser



from django.core.cache import cache

from swap_user.to_phone_otp.helpers import generate_otp_cache_key
from swap_user.to_phone_otp.models import PhoneOTPUser


class PhoneOTPAuthBackend:
    def authenticate(self, request, phone: str, otp_from_user: str, **credentials):
        """
        Authenticates used based on OTP (One Time Password) because current user model
        doesn't have a `password` field.

        Reference - https://docs.djangoproject.com/en/dev/topics/auth/customizing/#writing-an-authentication-backend
        """

        try:
            user = PhoneOTPUser.objects.get(phone=phone)
            user_id = user.id

            cache_key = generate_otp_cache_key(user_id)
            otp_value_from_cache = cache.get(cache_key)

            if otp_value_from_cache != otp_from_user:
                return None

            return user
        except PhoneOTPUser.DoesNotExist:
            return None

    def get_user(self, user_id: int):
        """
        Returns user instance based on PK or None.

        Reference - https://docs.djangoproject.com/en/dev/topics/auth/customizing/#writing-an-authentication-backend
        """

        try:
            user = PhoneOTPUser.objects.get(pk=user_id)

            return user
        except PhoneOTPUser.DoesNotExist:
            return None



import secrets

from swap_user.settings import swap_user_settings


def generate_otp():
    """
    Private method that generates OTP (One Time Password).
    """

    otp_alphabet = swap_user_settings.OTP_ALPHABET
    otp_length = swap_user_settings.OTP_LENGTH

    otp_digits = [secrets.choice(otp_alphabet) for _ in range(otp_length)]
    otp = "".join(otp_digits)

    return otp


def generate_otp_cache_key(user_id):
    """
    Generates cache key for storing OTP (One Time Password) per user.
    """

    cache_key = swap_user_settings.OTP_PATTERN.format(user_id=user_id)

    return cache_key
